@@ -10,7 +10,8 @@ from PyQt5.QtCore import pyqtSlot, QTimer, Qt, pyqtSignal, QItemSelection, QItem
 from PyQt5.QtGui import QContextMenuEvent, QIcon
 from PyQt5.QtWidgets import QMessageBox, QAbstractItemView, QUndoStack, QMenu, QWidget, QHeaderView
 
-from urh import constants
+from urh import settings
+from urh.awre import AutoAssigner
 from urh.controller.dialogs.MessageTypeDialog import MessageTypeDialog
 from urh.controller.dialogs.ProtocolLabelDialog import ProtocolLabelDialog
 from urh.models.LabelValueTableModel import LabelValueTableModel
@@ -29,6 +30,7 @@ from urh.ui.delegates.ComboBoxDelegate import ComboBoxDelegate
 from urh.ui.delegates.MessageTypeButtonDelegate import MessageTypeButtonDelegate
 from urh.ui.ui_analysis import Ui_TabAnalysis
 from urh.util import FileOperator, util
+from urh.util.Errors import Errors
 from urh.util.Formatter import Formatter
 from urh.util.Logger import logger
 from urh.util.ProjectManager import ProjectManager
@@ -84,7 +86,7 @@ class CompareFrameController(QWidget):
         self.assign_message_type_action.setChecked(True)
         self.assign_labels_action = self.analyze_menu.addAction(self.tr("Assign labels"))
         self.assign_labels_action.setCheckable(True)
-        self.assign_labels_action.setChecked(False)
+        self.assign_labels_action.setChecked(True)
         self.assign_participant_address_action = self.analyze_menu.addAction(self.tr("Assign participant addresses"))
         self.assign_participant_address_action.setCheckable(True)
         self.assign_participant_address_action.setChecked(True)
@@ -136,6 +138,8 @@ class CompareFrameController(QWidget):
 
         self.message_type_table_model.update()
 
+        self.tab_bar_collapsed = False
+
     # region properties
 
     @property
@@ -316,6 +320,8 @@ class CompareFrameController(QWidget):
         self.proto_tree_model.group_deleted.connect(self.on_group_deleted)
         self.proto_tree_model.proto_to_group_added.connect(self.on_proto_to_group_added)
 
+        self.ui.tabWidget.tabBarDoubleClicked.connect(self.on_tab_bar_double_clicked)
+
     def get_message_type_for_label(self, lbl: ProtocolLabel) -> MessageType:
         return next((msg_type for msg_type in self.proto_analyzer.message_types if lbl in msg_type), None)
 
@@ -348,6 +354,7 @@ class CompareFrameController(QWidget):
 
             self.show_all_cols()
 
+            self.ui.cbDecoding.setToolTip(self.ui.cbDecoding.currentText())
             for msg in messages:
                 msg.decoder = decoding
 
@@ -389,10 +396,13 @@ class CompareFrameController(QWidget):
         update = False
 
         for msg in self.proto_analyzer.messages:
-            i = next((i for i, d in enumerate(self.decodings) if d.name == msg.decoder.name), 0)
-            if msg.decoder != self.decodings[i]:
+            decoder = next((d for d in self.decodings if d.name == msg.decoder.name), None)
+            if decoder is None:
+                continue
+
+            if msg.decoder != decoder:
                 update = True
-                msg.decoder = self.decodings[i]
+                msg.decoder = decoder
                 msg.clear_decoded_bits()
                 msg.clear_encoded_bits()
 
@@ -412,6 +422,7 @@ class CompareFrameController(QWidget):
 
         self.ui.cbDecoding.addItem("...")
         self.ui.cbDecoding.setCurrentIndex(prev_index)
+        self.ui.cbDecoding.setToolTip(self.ui.cbDecoding.currentText())
         self.ui.cbDecoding.blockSignals(False)
 
     def add_protocol(self, protocol: ProtocolAnalyzer, group_id: int = 0) -> ProtocolAnalyzer:
@@ -440,6 +451,15 @@ class CompareFrameController(QWidget):
                         messsage_type.name += " (" + os.path.split(filename)[1].rstrip(".xml").rstrip(".proto") + ")"
                     self.proto_analyzer.message_types.append(messsage_type)
 
+        update_project = False
+        for msg in pa.messages:
+            if msg.participant is not None and msg.participant not in self.project_manager.participants:
+                self.project_manager.participants.append(msg.participant)
+                update_project = True
+
+        if update_project:
+            self.project_manager.project_updated.emit()
+
         self.message_type_table_model.update()
         self.add_protocol(protocol=pa)
 
@@ -455,7 +475,7 @@ class CompareFrameController(QWidget):
             self.refresh()
 
     def add_protocol_label(self, start: int, end: int, messagenr: int, proto_view: int, edit_label_name=True):
-        # Ensure atleast one Group is active
+        # Ensure at least one Group is active
         start, end = self.proto_analyzer.convert_range(start, end, proto_view, 0, decoded=True, message_indx=messagenr)
         message_type = self.proto_analyzer.messages[messagenr].message_type
         try:
@@ -512,7 +532,7 @@ class CompareFrameController(QWidget):
         # self.protocol_undo_stack.clear()
         self.proto_analyzer.messages[:] = []
         self.rows_for_protocols.clear()
-        align_labels = constants.SETTINGS.value("align_labels", True, bool)
+        align_labels = settings.read("align_labels", True, bool)
         line = 0
         first_msg_indices = []
         prev_line = 0
@@ -581,7 +601,7 @@ class CompareFrameController(QWidget):
                     pass
 
         for line in first_msg_indices:
-            self.ui.tblViewProtocol.setRowHeight(line, constants.SEPARATION_ROW_HEIGHT)
+            self.ui.tblViewProtocol.setRowHeight(line, settings.SEPARATION_ROW_HEIGHT)
 
         self.protocol_model.first_messages = first_msg_indices[:]
 
@@ -946,11 +966,15 @@ class CompareFrameController(QWidget):
 
         view = self.ui.cbProtoView.currentIndex()
         result = []
+        f = 1 if self.ui.cbProtoView.currentIndex() == 0 else 4 if self.ui.cbProtoView.currentIndex() == 1 else 8
         for i in range(row_start, row_end):
             message = self.proto_analyzer.messages[i]
             for label in message.message_type:
+                if label in result:
+                    continue
                 lbl_start, lbl_end = message.get_label_range(lbl=label, view=view, decode=True)
-                if any(j in range(lbl_start, lbl_end) for j in range(col_start, col_end)) and label not in result:
+                a = message.alignment_offset
+                if any(j in range(lbl_start, lbl_end) for j in range(col_start - a//f, col_end - a//f)):
                     result.append(label)
 
         return result
@@ -1007,7 +1031,7 @@ class CompareFrameController(QWidget):
 
         if self.assign_participants_action.isChecked():
             for protocol in self.protocol_list:
-                protocol.auto_assign_participants(self.protocol_model.participants)
+                AutoAssigner.auto_assign_participants(protocol.messages, self.protocol_model.participants)
             self.refresh_assigned_participants_ui()
 
         self.ui.progressBarLogicAnalyzer.setFormat("%p% (Assign message type by rules)")
@@ -1020,16 +1044,20 @@ class CompareFrameController(QWidget):
         self.ui.progressBarLogicAnalyzer.setValue(75)
 
         if self.assign_labels_action.isChecked():
-            self.proto_analyzer.auto_assign_labels()
-            self.protocol_model.update()
-            self.label_value_model.update()
-            self.message_type_table_model.update()
-            self.ui.tblViewMessageTypes.clearSelection()
+            try:
+                self.proto_analyzer.auto_assign_labels()
+                self.protocol_model.update()
+                self.label_value_model.update()
+                self.message_type_table_model.update()
+                self.ui.tblViewMessageTypes.clearSelection()
+            except Exception as e:
+                Errors.exception(e)
 
         self.ui.progressBarLogicAnalyzer.setValue(90)
 
         if self.assign_participant_address_action.isChecked():
-            self.proto_analyzer.auto_assign_participant_addresses(self.protocol_model.participants)
+            AutoAssigner.auto_assign_participant_addresses(self.proto_analyzer.messages,
+                                                           self.protocol_model.participants)
 
         self.ui.progressBarLogicAnalyzer.setValue(100)
         self.unsetCursor()
@@ -1079,6 +1107,7 @@ class CompareFrameController(QWidget):
     def on_combobox_decoding_current_index_changed(self):
         new_index = self.ui.cbDecoding.currentIndex()
         if new_index == -1:
+            self.ui.cbDecoding.setToolTip("")
             return
 
         if new_index == self.ui.cbDecoding.count() - 1:
@@ -1382,7 +1411,7 @@ class CompareFrameController(QWidget):
 
     @pyqtSlot(int)
     def on_ref_index_changed(self, new_ref_index: int):
-        if new_ref_index != -1:
+        if new_ref_index != -1 and self.protocol_model.row_count:
             hide_correction = 0
             for i in range(0, self.protocol_model.row_count):
                 if self.ui.tblViewProtocol.isRowHidden((new_ref_index + i) % self.protocol_model.row_count):
@@ -1463,3 +1492,16 @@ class CompareFrameController(QWidget):
     def on_message_type_name_edited(self, new_name: str):
         if self.ui.lblLabelValues.text().startswith("Labels of"):
             self.ui.lblLabelValues.setText("Labels of {}".format(new_name))
+
+    @pyqtSlot(int)
+    def on_tab_bar_double_clicked(self, index: int):
+        if not self.tab_bar_collapsed:
+            self.ui.tab_participants.hide()
+            self.ui.tab_protocols.hide()
+            self.ui.tabWidget.setMaximumHeight(self.ui.tabWidget.tabBar().height())
+            self.tab_bar_collapsed = True
+        else:
+            self.ui.tab_participants.show()
+            self.ui.tab_protocols.show()
+            self.ui.tabWidget.setMaximumHeight(9000)
+            self.tab_bar_collapsed = False
