@@ -1,10 +1,11 @@
 from statistics import median
 
+import numpy as np
 from PyQt5.QtCore import QRegExp, pyqtSlot, pyqtSignal
 from PyQt5.QtGui import QRegExpValidator, QIcon
 from PyQt5.QtWidgets import QWidget, QSpinBox, QLabel, QComboBox, QSlider
 
-from urh import constants
+from urh import settings
 from urh.dev import config
 from urh.dev.BackendHandler import BackendHandler, Backends
 from urh.dev.VirtualDevice import VirtualDevice
@@ -37,8 +38,11 @@ class DeviceSettingsWidget(QWidget):
         if self.is_rx:
             self.ui.spinBoxNRepeat.hide()
             self.ui.labelNRepeat.hide()
+        else:
+            self.ui.labelDCCorrection.hide()
+            self.ui.checkBoxDCCorrection.hide()
 
-        self.bw_sr_are_locked = constants.SETTINGS.value("lock_bandwidth_sample_rate", True, bool)
+        self.bw_sr_are_locked = settings.read("lock_bandwidth_sample_rate", True, bool)
         self.ui.cbDevice.clear()
         items = self.get_devices_for_combobox(continuous_send_mode)
         self.ui.cbDevice.addItems(items)
@@ -76,25 +80,32 @@ class DeviceSettingsWidget(QWidget):
         set_val(self.ui.spinBoxFreq, "frequency", config.DEFAULT_FREQUENCY)
         set_val(self.ui.spinBoxSampleRate, "sample_rate", config.DEFAULT_SAMPLE_RATE)
         set_val(self.ui.spinBoxBandwidth, "bandwidth", config.DEFAULT_BANDWIDTH)
-        set_val(self.ui.spinBoxGain, self.rx_tx_prefix+"gain", config.DEFAULT_GAIN)
-        set_val(self.ui.spinBoxIFGain, self.rx_tx_prefix+"if_gain", config.DEFAULT_IF_GAIN)
-        set_val(self.ui.spinBoxBasebandGain, self.rx_tx_prefix+"baseband_gain", config.DEFAULT_BB_GAIN)
+        set_val(self.ui.spinBoxGain, self.rx_tx_prefix + "gain", config.DEFAULT_GAIN)
+        set_val(self.ui.spinBoxIFGain, self.rx_tx_prefix + "if_gain", config.DEFAULT_IF_GAIN)
+        set_val(self.ui.spinBoxBasebandGain, self.rx_tx_prefix + "baseband_gain", config.DEFAULT_BB_GAIN)
         set_val(self.ui.spinBoxFreqCorrection, "freq_correction", config.DEFAULT_FREQ_CORRECTION)
-        set_val(self.ui.spinBoxNRepeat, "num_sending_repeats",
-                constants.SETTINGS.value('num_sending_repeats', 1, type=int))
+        set_val(self.ui.spinBoxNRepeat, "num_sending_repeats", settings.read('num_sending_repeats', 1, type=int))
 
-        if self.rx_tx_prefix+"antenna_index" in conf_dict:
-            self.ui.comboBoxAntenna.setCurrentIndex(conf_dict[self.rx_tx_prefix+"antenna_index"])
+        self.ui.lineEditSubdevice.setText(conf_dict.get("subdevice", ""))
 
-        if self.rx_tx_prefix+"gain" not in conf_dict:
+        if self.rx_tx_prefix + "antenna_index" in conf_dict:
+            self.ui.comboBoxAntenna.setCurrentIndex(conf_dict[self.rx_tx_prefix + "antenna_index"])
+
+        if self.rx_tx_prefix + "gain" not in conf_dict:
             self.set_default_rf_gain()
 
-        if self.rx_tx_prefix+"if_gain" not in conf_dict:
+        if self.rx_tx_prefix + "if_gain" not in conf_dict:
             self.set_default_if_gain()
 
-        if self.rx_tx_prefix+"baseband_gain" not in conf_dict:
+        if self.rx_tx_prefix + "baseband_gain" not in conf_dict:
             self.set_default_bb_gain()
 
+        if self.is_rx:
+            checked = conf_dict.get("apply_dc_correction", True)
+            if isinstance(checked, str):
+                checked = True if checked == "True" else False
+            self.ui.checkBoxDCCorrection.setChecked(checked)
+
         self.emit_editing_finished_signals()
 
     @property
@@ -134,6 +145,7 @@ class DeviceSettingsWidget(QWidget):
         self.ui.spinBoxBandwidth.editingFinished.connect(self.on_spinbox_bandwidth_editing_finished)
         self.ui.spinBoxPort.editingFinished.connect(self.on_spinbox_port_editing_finished)
         self.ui.lineEditIP.editingFinished.connect(self.on_line_edit_ip_editing_finished)
+        self.ui.lineEditSubdevice.editingFinished.connect(self.on_line_edit_subdevice_editing_finished)
 
         self.ui.comboBoxAntenna.currentIndexChanged.connect(self.on_combobox_antenna_current_index_changed)
         self.ui.comboBoxChannel.currentIndexChanged.connect(self.on_combobox_channel_current_index_changed)
@@ -147,7 +159,10 @@ class DeviceSettingsWidget(QWidget):
         self.ui.btnLockBWSR.clicked.connect(self.on_btn_lock_bw_sr_clicked)
 
         self.ui.btnRefreshDeviceIdentifier.clicked.connect(self.on_btn_refresh_device_identifier_clicked)
-        self.ui.comboBoxDeviceIdentifier.currentIndexChanged.connect(self.on_combo_box_device_identifier_current_index_changed)
+        self.ui.comboBoxDeviceIdentifier.currentIndexChanged.connect(
+            self.on_combo_box_device_identifier_current_index_changed)
+
+        self.ui.checkBoxDCCorrection.clicked.connect(self.on_check_box_dc_correction_clicked)
 
     def set_gain_defaults(self):
         self.set_default_rf_gain()
@@ -159,7 +174,7 @@ class DeviceSettingsWidget(QWidget):
         prefix = self.rx_tx_prefix
         if prefix + "rf_gain" in conf:
             key = prefix + "rf_gain"
-            gain = conf[key][int(median(range(len(conf[key]))))]
+            gain = conf[key][int(np.percentile(range(len(conf[key])), 25))]
             self.ui.spinBoxGain.setValue(gain)
 
     def set_default_if_gain(self):
@@ -175,7 +190,7 @@ class DeviceSettingsWidget(QWidget):
         prefix = self.rx_tx_prefix
         if prefix + "baseband_gain" in conf:
             key = prefix + "baseband_gain"
-            baseband_gain = conf[key][int(median(range(len(conf[key]))))]
+            baseband_gain = conf[key][int(np.percentile(list(range(len(conf[key]))), 25))]
             self.ui.spinBoxBasebandGain.setValue(baseband_gain)
 
     def sync_gain_sliders(self):
@@ -201,7 +216,7 @@ class DeviceSettingsWidget(QWidget):
                     spinbox.setSingleStep(conf[key][1] - conf[key][0])
                     spinbox.auto_update_step_size = False
                     if "default_" + key in conf:
-                        spinbox.setValue(conf["default_"+key])
+                        spinbox.setValue(conf["default_" + key])
                 else:
                     spinbox.setMinimum(conf[key].start)
                     spinbox.setMaximum(conf[key].stop)
@@ -213,6 +228,10 @@ class DeviceSettingsWidget(QWidget):
 
         self.ui.btnLockBWSR.setVisible("sample_rate" in conf and "bandwidth" in conf)
 
+        if self.device is not None:
+            self.ui.labelSubdevice.setVisible(self.device.subdevice is not None)
+            self.ui.lineEditSubdevice.setVisible(self.device.subdevice is not None)
+
         if "freq_correction" in conf:
             self.ui.labelFreqCorrection.setVisible(True)
             self.ui.spinBoxFreqCorrection.setVisible(True)
@@ -282,6 +301,9 @@ class DeviceSettingsWidget(QWidget):
         self.ui.labelIP.setVisible("ip" in conf)
         self.ui.spinBoxPort.setVisible("port" in conf)
         self.ui.labelPort.setVisible("port" in conf)
+        show_dc_correction = self.is_rx and self.device is not None and self.device.apply_dc_correction is not None
+        self.ui.checkBoxDCCorrection.setVisible(show_dc_correction)
+        self.ui.labelDCCorrection.setVisible(show_dc_correction)
 
     def get_devices_for_combobox(self, continuous_send_mode):
         items = []
@@ -324,14 +346,16 @@ class DeviceSettingsWidget(QWidget):
         self.ui.spinBoxSampleRate.editingFinished.emit()
         self.ui.spinBoxFreqCorrection.editingFinished.emit()
         self.ui.lineEditIP.editingFinished.emit()
+        self.ui.lineEditSubdevice.editingFinished.emit()
         self.ui.spinBoxPort.editingFinished.emit()
         self.ui.comboBoxAntenna.currentIndexChanged.emit(self.ui.comboBoxAntenna.currentIndex())
         self.ui.comboBoxChannel.currentIndexChanged.emit(self.ui.comboBoxChannel.currentIndex())
+        self.ui.checkBoxDCCorrection.clicked.emit(self.ui.checkBoxDCCorrection.isChecked())
 
     def emit_device_parameters_changed(self):
         settings = {"name": str(self.device.name)}
         for attrib in ("frequency", "sample_rate", "bandwidth", "gain", "if_gain", "baseband_gain", "freq_correction",
-                       "antenna_index", "num_sending_repeats"):
+                       "antenna_index", "num_sending_repeats", "apply_dc_correction", "subdevice"):
             try:
                 value = getattr(self.device, attrib, None)
                 if value is not None:
@@ -346,7 +370,7 @@ class DeviceSettingsWidget(QWidget):
     @pyqtSlot()
     def on_btn_lock_bw_sr_clicked(self):
         self.bw_sr_are_locked = self.ui.btnLockBWSR.isChecked()
-        constants.SETTINGS.setValue("lock_bandwidth_sample_rate", self.bw_sr_are_locked)
+        settings.write("lock_bandwidth_sample_rate", self.bw_sr_are_locked)
         if self.bw_sr_are_locked:
             self.ui.btnLockBWSR.setIcon(QIcon(":/icons/icons/lock.svg"))
             self.ui.spinBoxBandwidth.setValue(self.ui.spinBoxSampleRate.value())
@@ -376,6 +400,10 @@ class DeviceSettingsWidget(QWidget):
     def on_line_edit_ip_editing_finished(self):
         self.device.ip = self.ui.lineEditIP.text()
 
+    @pyqtSlot()
+    def on_line_edit_subdevice_editing_finished(self):
+        self.device.subdevice = self.ui.lineEditSubdevice.text()
+
     @pyqtSlot()
     def on_spinbox_port_editing_finished(self):
         self.device.port = self.ui.spinBoxPort.value()
@@ -480,12 +508,17 @@ class DeviceSettingsWidget(QWidget):
         self.ui.comboBoxDeviceIdentifier.clear()
         self.ui.comboBoxDeviceIdentifier.addItems(self.device.get_device_list())
 
+    @pyqtSlot(bool)
+    def on_check_box_dc_correction_clicked(self, checked: bool):
+        self.device.apply_dc_correction = bool(checked)
+
     @pyqtSlot()
     def on_combo_box_device_identifier_current_index_changed(self):
         if self.device is not None:
             self.device.device_serial = self.ui.comboBoxDeviceIdentifier.currentText()
             self.device.device_number = self.ui.comboBoxDeviceIdentifier.currentIndex()
 
+
 if __name__ == '__main__':
     from PyQt5.QtWidgets import QApplication
     from urh.controller.MainController import MainController
